@@ -1,10 +1,13 @@
 import asyncio
 import builtins
+import ctypes
 import inspect
 import json
+import os
+import random
 import threading
 import traceback
-from collections import deque
+from collections import defaultdict, deque
 from contextlib import contextmanager
 from contextvars import ContextVar
 from itertools import count
@@ -15,6 +18,81 @@ from .registry import callback_registry
 
 _c = count(1)
 
+
+class ThreadKilledException(Exception):
+    pass
+
+
+class KillableThread(threading.Thread):
+    @property
+    def dead(self):
+        return not self.is_alive() or getattr(self, "_dead", False)
+
+    def kill(self):
+        ctypes.pythonapi.PyThreadState_SetAsyncExc(
+            ctypes.c_long(self.ident), ctypes.py_object(ThreadKilledException)
+        )
+
+
+class NamedThreads:
+    @classmethod
+    def current(cls):
+        thread = threading.current_thread()
+        if isinstance(thread, KillableThread):
+            return thread
+        else:
+            return None
+
+    def __init__(self):
+        self.words = [
+            word
+            for word in open(
+                os.path.join(os.path.dirname(__file__), "words.txt")
+            )
+            .read()
+            .split("\n")
+            if word
+        ]
+        random.shuffle(self.words)
+        self.threads = {}
+        self.count = count(1)
+
+    def run_in_thread(self, fn, session):
+        def run():
+            reason = " finished"
+            self.threads[word] = thread
+            with session.set_context():
+                with new_evalid():
+                    session.queue_result(
+                        H.div("Starting thread ", H.strong(word)), type="info",
+                    )
+                    try:
+                        fn()
+                    except ThreadKilledException:
+                        reason = " killed"
+                    except Exception as e:
+                        session.queue_result(e, type="exception")
+                    finally:
+                        thread._dead = True
+                        session._clean_owners()
+                        del self.threads[word]
+                        self.words.append(word)
+                        session.queue_result(
+                            H.div("Thread ", H.strong(word), reason),
+                            type="info",
+                        )
+
+        thread = KillableThread(target=run, daemon=True)
+
+        if not self.words:
+            self.words.append(f"t{next(self.count)}")
+        word = self.words.pop()
+
+        thread.start()
+        return word, thread
+
+
+threads = NamedThreads()
 _current_session = ContextVar("current_session", default=None)
 _current_print_session = ContextVar("current_print_session", default=None)
 _current_evalid = ContextVar("current_evalid", default=None)
@@ -48,7 +126,9 @@ class Session:
         self.last_nav = ""
         self.in_queue = deque()
         self.out_queue = deque()
-        self.semaphore = threading.Semaphore(value=0)
+        self.semaphores = defaultdict(lambda: threading.Semaphore(value=0))
+        self.navs = {}
+        self.owners = []
         self._token = None
         self._tokenp = None
 
@@ -56,6 +136,42 @@ class Session:
     # Utilities #
     #############
 
+    @property
+    def owner(self):
+        return self.owners[-1] if self.owners else None
+
+    def _clean_owners(self):
+        while self.owners and (curr := self.owners[-1]) and curr.dead:
+            del self.navs[curr]
+            self.pop_owner()
+
+    def _acquire_all(self):
+        current = self.semaphores[self.owner]
+        while current.acquire(blocking=False):
+            pass
+
+    def _release_all(self):
+        current = self.semaphores[self.owner]
+        if self.in_queue:
+            current.release(len(self.in_queue))
+
+    def _current_prompt(self):
+        for act in self.navs.get(self.owner, []):
+            act()
+
+    def push_owner(self, thread):
+        self._acquire_all()
+        self.owners.append(thread)
+        self._release_all()
+        self._current_prompt()
+
+    def pop_owner(self):
+        self._acquire_all()
+        if self.owners:
+            self.owners.pop()
+        self._release_all()
+        self._current_prompt()
+
     def enter(self, capture_print=True):
         self._token = _current_session.set(self)
         if capture_print:
@@ -121,25 +237,29 @@ class Session:
     ################
 
     def set_prompt(self, prompt):
-        if prompt != self.last_prompt:
-            self.queue(command="set_mode", html=prompt)
+        self.add_nav_action(lambda: self.queue(command="set_mode", html=prompt))
 
     def set_nav(self, nav):
-        if nav != self.last_nav:
-            self.queue(command="set_nav", value=hrepr(nav))
-            self.last_nav = nav
+        self.add_nav_action(
+            lambda: self.queue(command="set_nav", value=hrepr(nav))
+        )
+
+    def add_nav_action(self, action):
+        self.navs.setdefault(NamedThreads.current(), []).append(action)
 
     @contextmanager
     def prompt(self, prompt="", nav=H.span()):
         self.set_prompt(prompt)
         self.set_nav(nav)
+        self._clean_owners()
+        self._current_prompt()
         expr = self.next()
         with self.set_context():
             with new_evalid():
                 yield expr
 
     def next(self):
-        self.semaphore.acquire()
+        self.semaphores[NamedThreads.current()].acquire()
         return self.in_queue.popleft()
 
     ###########
@@ -206,7 +326,7 @@ class Session:
 
     def submit(self, data):
         self.in_queue.append(data)
-        self.semaphore.release()
+        self.semaphores[self.owner].release()
 
     async def recv(self, **command):
         cmd = command.pop("command", "none")
